@@ -60,10 +60,13 @@ Q_DECLARE_FLAGS(RenderFlags, RenderFlag)
 class TILEDSHARED_EXPORT MapRenderer
 {
 public:
+    enum CellType {
+        OrthogonalCells,
+        HexagonalCells
+    };
+
     MapRenderer(const Map *map)
         : mMap(map)
-        , mObjectLineWidth(2)
-        , mPainterScale(1)
     {}
 
     virtual ~MapRenderer();
@@ -132,7 +135,7 @@ public:
     virtual void drawGrid(QPainter *painter, const QRectF &rect,
                           QColor gridColor = Qt::black) const = 0;
 
-    typedef std::function<void(const Cell &, const QPointF &, const QSizeF &)> RenderTileCallback;
+    typedef std::function<void(QPoint, const QPointF &)> RenderTileCallback;
 
     /**
      * Draws the given \a layer using the given \a painter.
@@ -140,18 +143,22 @@ public:
      * Optionally, you can pass in the \a exposed rect (of pixels), so that
      * only tiles that can be visible in this area will be drawn.
      */
-    virtual void drawTileLayer(QPainter *painter, const TileLayer *layer,
-                               const QRectF &exposed = QRectF()) const = 0;
+    void drawTileLayer(QPainter *painter, const TileLayer *layer,
+                       const QRectF &exposed = QRectF()) const;
 
     /**
-     * Draws the given \a layer using the given \a renderTile callback.
+     * Calls the given \a renderTile callback for each tile in the given
+     * \a exposed rectangle.
      *
-     * Optionally, you can pass in the \a exposed rect (of pixels), so that
-     * only tiles that can be visible in this area will be drawn.
+     * The callback takes two arguments:
+     *
+     * \list
+     * \li \c tilePos - The tile position of the cell being rendered.
+     * \li \c screenPos - The screen position of the cell being rendered.
+     * \endlist
      */
-    virtual void drawTileLayer(const TileLayer *layer,
-                               const RenderTileCallback &renderTile,
-                               const QRectF &exposed = QRectF()) const = 0;
+    virtual void drawTileLayer(const RenderTileCallback &renderTile,
+                               const QRectF &exposed) const = 0;
 
     /**
      * Draws the tile selection given by \a region in the specified \a color.
@@ -181,7 +188,7 @@ public:
      */
     void drawImageLayer(QPainter *painter,
                         const ImageLayer *imageLayer,
-                        const QRectF &exposed = QRectF());
+                        const QRectF &exposed = QRectF()) const;
 
     /**
      * Returns the tile coordinates matching the given pixel position.
@@ -258,17 +265,22 @@ public:
     RenderFlags flags() const { return mFlags; }
     void setFlags(RenderFlags flags) { mFlags = flags; }
 
+    CellType cellType() const { return mCellType; }
+
     static QPolygonF lineToPolygon(const QPointF &start, const QPointF &end);
 
 protected:
-    QPen makeGridPen(const QPaintDevice *device, QColor color) const;
+    static QPen makeGridPen(const QPaintDevice *device, QColor color);
+
+    void setCellType(CellType cellType) { mCellType = cellType; }
 
 private:
     const Map *mMap;
 
     RenderFlags mFlags;
-    qreal mObjectLineWidth;
-    qreal mPainterScale;
+    CellType mCellType = OrthogonalCells;
+    qreal mObjectLineWidth = 2;
+    qreal mPainterScale = 1;
 };
 
 inline const Map *MapRenderer::map() const
@@ -308,13 +320,8 @@ public:
         BottomLeft
     };
 
-    enum CellType {
-        OrthogonalCells,
-        HexagonalCells
-    };
-
     explicit CellRenderer(QPainter *painter, const MapRenderer *renderer,
-                          const QColor &tintColor, CellType cellType = OrthogonalCells);
+                          const QColor &tintColor);
 
     ~CellRenderer() { flush(); }
 
@@ -330,7 +337,6 @@ private:
     const Tile *mTile;
     QVector<QPainter::PixmapFragment> mFragments;
     const bool mIsOpenGL;
-    const CellType mCellType;
     const QColor mTintColor;
 };
 
